<?php


namespace CubiStore\Web\Utils;

use SimpleXMLElement;

class Apk
{
    /**
     * @var string
     */
    private $file;
    /**
     * @var SimpleXMLElement|null
     */
    private $manifest;

    /**
     * Apk constructor.
     * @param string $file
     */
    public function __construct(string $file)
    {
        $this->file = $file;
    }

    public function isValid(): bool
    {
        // Get first 4 bytes
        $fh = fopen($this->file, 'r');
        $header = fread($fh, 4);
        fclose($fh);

        // Check if uploaded file is a valid ZIP file
        if ($header !== "PK\x03\x04") {
            return false;
        }

        $zip = new \ZipArchive();
        $zip->open($this->file);
        $valid = $zip->statName("AndroidManifest.xml") !== false && $zip->statName("resources.arsc");
        $zip->close();

        return $valid;
    }

    public function getManifest(): SimpleXMLElement
    {
        if ($this->manifest === null) {
            $xml = $this->cmd('androguard', 'axml', '-o', '/dev/stdout', $this->file);
            $this->manifest = simplexml_load_string($xml);
        }

        return $this->manifest;
    }

    private function getAttribute(string $name, ?string $namespace = null): string
    {
        if ($namespace !== null) {
            return (string)$this->getManifest()->attributes($namespace, true)[$name];
        }

        return (string)$this->getManifest()[$name];
    }

    public function getPackageName(): string
    {
        return $this->getAttribute('package');
    }

    public function getVersionCode(): int
    {
        return intval($this->getAttribute('versionCode', 'android'), 10);
    }

    public function getVersionName(): string
    {
        return $this->getAttribute('versionName', 'android');
    }

    public function getLabel(): string
    {
        $code = (string)$this->getManifest()->application->attributes('android', true)['label'];
        $output = $this->cmd('androguard', 'arsc', '--id', $code, $this->file);
        preg_match(':\<default\> = \'(.*)\'$:m', $output, $matches);
        return stripslashes($matches[1]);
    }

    private function cmd(string $cmd, string... $args): string
    {
        $proc = popen($cmd . ' ' . implode(' ', array_map('escapeshellarg', $args)), 'r');
        $data = '';
        while ($text = fread($proc, 1024)) {
            $data .= $text;
        }
        pclose($proc);
        return $data;
    }
}
